#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "uart.h"

#define BAUDRATE 19200L
#define XTAL 16000000L

volatile uint8_t leds[24];

uint8_t hextoint(char);

void SendString(char* text)
{
	uint8_t i = 0;
	
	while (text[i] != 0) 
	{
		sendchar(text[i]);
		i++;
	}
}


void InitTimer(void)
{
	TCCR0 = _BV(CS00);   // CPU-Takt/256
	TIMSK = _BV(TOIE0);
}

volatile uint32_t timerticks;

void WaitTimerTicks(uint32_t ticks)
{
	timerticks = 0;
	while (timerticks <= ticks);
}


int main (void)
{
	static uint8_t flag = 0;

	USART_Init(UART_BAUD_SELECT(BAUDRATE,XTAL),UARTSINGLE); 	// single speed
	
	SendString("ASCII-Interface\n");
	
	InitTimer();
	
    DDRA = 0xff;
	DDRB = 0xff;
	DDRC = 0xff;
	DDRD = 0xff;
	
	InitTimer();
	sei();
	
	PORTA = 0xff;
	PORTB = 0xff;
	PORTC = 0xff;
	PORTD = 0xff;
   /* for (;;) 
	{
	}*/ 

    for (;;) 
	{
		uint8_t c = 0;
		uint8_t led = 0;
		
		char *input = recline();
		SendString(input);
		sendchar('\n');
		
		switch(input[c++]) {
		case 's':
		case 'S':
			while(input[c] && input[c+1]) {
				leds[led++] = hextoint(input[c])*16 + hextoint(input[c+1]);
				c+=2;
			}
			break;
		case 'c':
		case 'C':
			for(uint8_t i = 0; i<24; ++i) {
				leds[i] = 0;
			}
			break;
		case 'f':
		case 'F':
			for(uint8_t i = 0; i<24; ++i) {
				leds[i] = 255;
			}
			break;
		case 'r':
		case 'R':
			if (!flag) {
				for(uint8_t i=0,f=0; i<24; ++i) {
					if (f) {
						leds[i]=0;
						++f;
					} else {
						leds[i]=255;
						--f;
					}
				}
				flag++;
			} else {
				for(uint8_t i=0,f=0; i<24; ++i) {
					if (!f) {
						leds[i]=0;
						--f;
					} else {
						leds[i]=255;
						++f;
					}
				}
				flag--;
			}
			break;
		default:
			SendString("Unknown command\n\n");
		}
			/*
				SendString("LED: ");
				char bff[5];
				itoa(i,bff,10);
				SendString(bff);
				if (i < 8)
				{
					PORTC = 0;
					PORTA = _BV(i);
				} else if (i < 16)
				{
					PORTA = 0;
					PORTB = _BV(i-8);
				} else
				{
					PORTB = 0;
					PORTC = _BV(i-16);
				}
				//WaitTimerTicks(40000);
				recchar();
				*/
	}
}

SIGNAL (SIG_OVERFLOW0) 
{
    /* Interrupt Code */
        static uint16_t ledticks;

        timerticks++;

        ledticks++;
        if (ledticks > 255)
        {
                ledticks = 0;
                PORTA = 0xFF;
                PORTB = 0xFF;
                PORTC = 0xFF;
        }

        for(uint8_t i = 0; i <= 7; i++)
        {
                if (leds[i] <= ledticks)
                {
                        PORTA &= ~_BV(i);
                }
                if (leds[i+8] <= ledticks)
                {
                        PORTB &= ~_BV(i);
                }
                if (leds[i+16] <= ledticks)
                {
                        PORTC &= ~_BV(i);
                }
        }
}

void delay_ms(uint16_t ms)
{
        for(uint16_t i = 0; i <= ms; i++)
        {
                _delay_ms(1);
        }
}

uint8_t hextoint(char c)
{
	if (c >= 'A' && c <= 'F') {
		return(c-'A'+10);
	} else if (c >= 'a' && c <= 'f') {
		return(c-'a'+10);
	} else if (c >= '0' && c <= '9') {
		return(c-'0');
	} else {
		return(0);
	}
}